package org.rcisoft.business.system.user.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.system.user.entity.SysUser;
import org.rcisoft.business.system.user.service.UserService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.ResultGenerator;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.support.JstlUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * @Author: GaoLiWei
 * @Date: Created in 10:462018/5/8
 */
@Controller
@RequestMapping("/login")
public class LoginController extends PaginationController<SysUser> {

    @Autowired
    private UserService userService;

    /**
     * 登录
     * @param user
     * @return
     */
    @ApiImplicitParams({@ApiImplicitParam(name = "user", value = "用户名,密码", required = true, dataType = "SysUser")})
    @PostMapping(value = "${jwt.route.authentication.path}")
    @ResponseBody
    public Result login(@RequestBody SysUser user){
        String username = user.getUserNm();
        String password = user.getUserPwd();
        final String token = userService.login(username, password);
        Result result = ResultGenerator.genSuccessResult(token);
        return result;
    }

    @ApiOperation(value = "修改密码")
    @ApiImplicitParams({@ApiImplicitParam(name = "newPassWord", value = "新密码", required = true, dataType = "varchar",paramType = "body"),
            @ApiImplicitParam(name = "oldPassWord", value = "旧密码", required = true, dataType = "varchar",paramType = "body")})
    @RequestMapping(value = "/updatePassWord")
    @ResponseBody
    public Result updatePassWord(@RequestParam String newPassWord, @RequestParam String oldPassWord){
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        return Result.builder(userService.updatePassWord(newPassWord, oldPassWord, userId));


    }


    @ApiOperation(value = "重置密码")
    @ResponseBody
    @PutMapping(value = "/resetPassWord")
    public Result resetPassWord(@RequestParam String needUserId){
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        int flag = userService.resetPassWord(userId, needUserId);
        return Result.builder(new PersistModel(flag));
    }

    @RequestMapping(value = "/loginWechat")
    @ResponseBody
    public Result loginWechat(@RequestBody SysUser user){
        return Result.builder(userService.loginWechat(user));
    }
    @RequestMapping(value = "/openIdBindingFlag")
    @ResponseBody
    public Object openIdBindingFlag(@RequestParam String openId){
        return userService.getUserByOpenId(openId);
    }





}
